import logging
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal.rest import rest_helpers
from portal.rest.schemas import worker_schema, workers_schema
from portal.service import auth, permissions
from portal.service.auth import find_client
from portal.service.workers import create_worker, delete_worker, find_all_workers, \
    update_worker
from portal.service.general import find_worker

workers_namespace = Namespace('workers')
log = logging.getLogger(__name__)


@workers_namespace.route('')
class WorkerList(Resource):
    @jwt_required
    # @workers_namespace.response(200, 'List of workers', model=workers_schema)
    @workers_namespace.response(403, 'Not allowed to list workers')
    def get(self):
        client = auth.find_client()
        # authorization
        permissions.require_sysadmin(client)
        workers_list = find_all_workers()
        return workers_schema.dump(workers_list)[0]

    @jwt_required
    # @workers_namespace.response(201, 'Created worker', model=worker_schema)
    @workers_namespace.response(403, 'Not allowed to add worker')
    def post(self):
        client = auth.find_client()
        # authorization
        permissions.require_sysadmin(client)

        data = rest_helpers.parse_request_data(
            worker_schema, action='create', resource='worker'
        )
        new_worker = create_worker(**data)
        return worker_schema.dump(new_worker)[0], 201


@workers_namespace.route('/<string:wid>')
@workers_namespace.param('wid', 'Worker id')
@workers_namespace.response(404, 'Worker not found')
class WorkerResource(Resource):

    @jwt_required
    # @workers_namespace.response(200, 'Worker', model=worker_schema)
    @workers_namespace.response(403, 'Not allowed to access worker')
    def get(self, wid: str):
        client = find_client()
        permissions.require_sysadmin(client)

        worker = find_worker(wid)
        return worker_schema.dump(worker)[0]

    @jwt_required
    @workers_namespace.response(204, 'Worker deleted')
    @workers_namespace.response(403, 'Not allowed to delete worker')
    def delete(self, wid: str):
        client = find_client()
        # authorization
        permissions.require_sysadmin(client)

        worker = find_worker(wid)
        delete_worker(worker)
        return '', 204

    @jwt_required
    @workers_namespace.response(204, 'Worker updated')
    @workers_namespace.response(403, 'Not allowed to update worker')
    def put(self, wid: str):
        client = find_client()
        # authorization
        permissions.require_sysadmin(client)

        data = rest_helpers.parse_request_data(
            worker_schema, action='update', resource='worker', partial=True
        )

        worker = find_worker(wid)
        update_worker(worker, data)
        return '', 204
