"""
Permissions service
"""

import logging
from typing import List

from portal.database.models import Course, Client, ClientType
from portal.service import general
from portal.service.errors import ForbiddenError
from portal.service.general import find_user

FILTER_PERMISSION_ATTRS = ['id', 'role_id', 'role',
                           'description', 'created_at', 'updated_at']

log = logging.getLogger(__name__)


def require_client(client: Client, course: Course, permissions=None):
    """Require clients permissions for a course

   Args:
       client(Client): Client instance
       course(Course): Course instance
       permissions: List of permissions required

   Returns(bool): True if checks has passed
   """
    if not check_client(client=client, course=course, permissions=permissions):
        raise ForbiddenError(client.id)


def check_client(client: Client, course: Course, permissions: List=None) -> bool:
    """Checks a client's permissions based on his roles in the course.
    Parameter <code>permissions</code> contains a list of lists.
    The check passes if the user has all of the permissions of at least one inner list.
    That means (simplified): user.permissionsInCourse.contain ((permA AND permB) OR (permC))

    Args:
        client(Client): User instance
        course(Course): Course instance
        permissions(List): Permissions list. Contains lists of permission names or permission names

    Returns(bool): True if the client has all required permission in the course
    """
    log.debug(
        f"[PERM] Client {client.id} in course {course.codename}: {permissions}")
    # admin privileges are bound to User, not a Permission
    if check_sysadmin(client):
        return True

    effective_permissions = effective_permissions_for_course(client, course=course)
    if not effective_permissions:
        return False

    res = any(__resolve_permissions(conjunction, effective_permissions) for conjunction in permissions)

    log.debug(f"[PERM] {permissions} for {client.id}: {res}")
    return res


def __resolve_permissions(permissions_required_all: list, effective_permissions: dict):
    if isinstance(permissions_required_all, str):
        permissions_required_all = [permissions_required_all]
    return all(effective_permissions.get(permission, False) for permission in permissions_required_all)


def check_sysadmin(client: Client) -> bool:
    if client.type == ClientType.USER:
        return client.is_admin
    return False


def require_sysadmin(client: Client):
    """Requires the client to be a User which is admin

    Args:
        client(Client): Client instance
    """
    if not check_sysadmin(client):
        raise ForbiddenError(client.id)


def require_any_check(client, checks: list):
    """Requires at least one check in the list of checks to pass
    Args:
        client:
        checks(list): List of checks
    Returns:
    """
    if not any(checks):
        raise ForbiddenError(client.id)


def get_effective_permissions(client: Client, course_id: str = None) -> dict:
    """Gets effective permissions for a client in course.
    If no course is specified, returns the client's
    effective permissions in all courses he is a part of.

    Args:
        client(Client): Client instance
        course_id(str): Course ID

    Returns(dict): Effective permissions for the client. Keys are course IDs, values
    dictionaries of permissions with their values.
    """
    course = general.find_course(course_id, throws=False)
    courses = [course] if course else client.courses
    return build_effective_permissions(client, *courses)


def build_effective_permissions(client: Client, *courses) -> dict:
    """Builds effective permissions in a list of courses for the client

    Args:
        client(Client): Client instance
        *courses: List of courses

    Returns(dict): Effective permissions
    """
    result = {}
    for course in courses:
        result[course.id] = effective_permissions_for_course(
            client, course=course)
    return result


def effective_permissions_for_course(client: Client, course: Course) -> dict:
    """Extracts effective permissions for a client in one course

    Args:
        client(Client): Client instance
        course(Course): Course instance

    Returns(dict): Effective permissions dictionary
    """
    permissions = client.get_permissions_for_course(course)
    result = {}
    for permission in permissions:
        for key, value in vars(permission).items():
            if not key.startswith("_") and key not in FILTER_PERMISSION_ATTRS:
                result[key] = result.get(key) or value
    log.debug(f"[PERM] Effective permissions: {client.id} in course {course.codename}: {result}")
    return result
